/************************************************************************************
iPhone Communication RT-Component
Copyright (c) 2010, Nagoya Institute of Technology
All rights reserved.

Contact us if you use this software for sell.
If you use this software not for sell, you can use this software under BSD lisence.
See the files LICENSE.TXT and LICENSE-BSD.TXT for more details.                     
************************************************************************************/

//package org.jsn.jdf.util;


public class ByteUtil
{

//    private ByteUtil()
//    {
//    }

    public byte[] short2byte(short s)
    {
        byte dest[] = new byte[2];
        dest[1] = (byte)(s & 0xff);
        dest[0] = (byte)(s >>> 8 & 0xff);
        return dest;
    }

    public byte[] int2byte(int i)
    {
        byte dest[] = new byte[4];
        dest[3] = (byte)(i & 0xff);
        dest[2] = (byte)(i >>> 8 & 0xff);
        dest[1] = (byte)(i >>> 16 & 0xff);
        dest[0] = (byte)(i >>> 24 & 0xff);
        return dest;
    }

    public byte[] long2byte(long l)
    {
        byte dest[] = new byte[8];
        dest[7] = (byte)(int)(l & 255L);
        dest[6] = (byte)(int)(l >>> 8 & 255L);
        dest[5] = (byte)(int)(l >>> 16 & 255L);
        dest[4] = (byte)(int)(l >>> 24 & 255L);
        dest[3] = (byte)(int)(l >>> 32 & 255L);
        dest[2] = (byte)(int)(l >>> 40 & 255L);
        dest[1] = (byte)(int)(l >>> 48 & 255L);
        dest[0] = (byte)(int)(l >>> 56 & 255L);
        return dest;
    }

    public byte[] float2byte(float f)
    {
        byte dest[] = new byte[4];
        return setfloat(dest,0,f);
    }
    public byte[] double2byte(double d)
    {
        byte dest[] = new byte[8];
        return setdouble(dest,0,d);
    }

    public byte getbyte(byte src[], int offset)
    {
        return src[offset];
    }

    public byte[] getbytes(byte src[], int offset, int length)
    {
        byte dest[] = new byte[length];
        System.arraycopy(src, offset, dest, 0, length);
        return dest;
    }

    public short getshort(byte src[], int offset)
    {
        return (short)((src[offset] & 0xff) << 8 | src[offset + 1] & 0xff);
    }

    public int getint(byte src[], int offset)
    {
        return 
            (src[offset] & 0xff) << 24 | 
            (src[offset + 1] & 0xff) << 16 | 
            (src[offset + 2] & 0xff) << 8 | 
            src[offset + 3] & 0xff;
    }

    public long getlong(byte src[], int offset)
    {
        return 
            (long)getint(src, offset) << 32 | 
            (long)getint(src, offset + 4) & 0xffffffffL;
    }

    public float getfloat(byte src[], int offset)
    {
        return Float.intBitsToFloat(getint(src, offset));
    }

    public double getdouble(byte src[], int offset)
    {
        return Double.longBitsToDouble(getlong(src, offset));
    }

    public static final byte[] setbyte(byte dest[], int offset, byte b)
    {
        dest[offset] = b;
        return dest;
    }

    public byte[] setbytes(byte dest[], int offset, byte src[])
    {
        System.arraycopy(src, 0, dest, offset, src.length);
        return dest;
    }

    public byte[] setbytes(byte dest[], int offset, byte src[], int len)
    {
        System.arraycopy(src, 0, dest, offset, len);
        return dest;
    }

    public byte[] setshort(byte dest[], int offset, short s)
    {
        dest[offset] = (byte)(s >>> 8 & 0xff);
        dest[offset + 1] = (byte)(s & 0xff);
        return dest;
    }

    public byte[] setint(byte dest[], int offset, int i)
    {
        dest[offset] = (byte)(i >>> 24 & 0xff);
        dest[offset + 1] = (byte)(i >>> 16 & 0xff);
        dest[offset + 2] = (byte)(i >>> 8 & 0xff);
        dest[offset + 3] = (byte)(i & 0xff);
        return dest;
    }

    public byte[] setlong(byte dest[], int offset, long l)
    {
        setint(dest, offset, (int)(l >>> 32));
        setint(dest, offset + 4, (int)(l & 0xffffffffL));
        return dest;
    }

    public byte[] setfloat(byte dest[], int offset, float f)
    {
        return setint(dest, offset, Float.floatToIntBits(f));
    }

    public byte[] setdouble(byte dest[], int offset, double d)
    {
        return setlong(dest, offset, Double.doubleToLongBits(d));
    }

    public boolean isEquals(byte b[], String s)
    {
        if(b == null || s == null)
            return false;
        int slen = s.length();
        if(b.length != slen)
            return false;
        for(int i = slen; i-- > 0;)
            if(b[i] != s.charAt(i))
                return false;

        return true;
    }

    public boolean isEquals(byte a[], byte b[])
    {
        if(a == null || b == null)
            return false;
        if(a.length != b.length)
            return false;
        for(int i = a.length; i-- > 0;)
            if(a[i] != b[i])
                return false;

        return true;
    }
}
