/************************************************************************************
iPhone Communication RT-Component
Copyright (c) 2010, Nagoya Institute of Technology
All rights reserved.

Contact us if you use this software for sell.
If you use this software not for sell, you can use this software under BSD lisence.
See the files LICENSE.TXT and LICENSE-BSD.TXT for more details.                     
************************************************************************************/
// -*- Java -*-
/*!
 * @file  iPhoneCommImpl.java
 * @brief Communicator for iPhone
 * @date  $Date$
 *
 * $Id$
 */

import RTC.TimedDoubleSeq;
import RTC.TimedShortSeq;
import RTC.TimedBooleanSeq;
import RTC.TimedString;
import RTC.TimedUShort;
import RTC.TimedShort;
import jp.go.aist.rtm.RTC.DataFlowComponentBase;
import jp.go.aist.rtm.RTC.Manager;
import jp.go.aist.rtm.RTC.port.InPort;
import jp.go.aist.rtm.RTC.port.OutPort;
import jp.go.aist.rtm.RTC.util.DataRef;
import jp.go.aist.rtm.RTC.util.StringHolder;
import jp.go.aist.rtm.RTC.util.IntegerHolder;
import RTC.ReturnCode_t;

import javax.vecmath.*;

import IIS.TimedVelocity2D;
/*!
 * @class iPhoneCommImpl
 * @brief Communicator for iPhone
 *
 */
public class iPhoneCommImpl extends DataFlowComponentBase {

  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
	public iPhoneCommImpl(Manager manager) {  
        super(manager);
        // <rtc-template block="initializer">
        m_robotPos_val = new TimedDoubleSeq(new RTC.Time(0,0), new double[3]);
        m_robotPos = new DataRef<TimedDoubleSeq>(m_robotPos_val);
        m_robotPos.v = new TimedDoubleSeq(new RTC.Time(0,0), new double[3]);
        m_posInIn = new InPort<TimedDoubleSeq>("posIn", m_robotPos);        

//        m_urg_val = new TimedShortSeq();
        m_urg_val = new TimedShortSeq(new RTC.Time(0,0), new short[180]);
        m_urg = new DataRef<TimedShortSeq>(m_urg_val);
        m_urg.v = new TimedShortSeq(new RTC.Time(0,0), new short[180]);
        m_urgInIn = new InPort<TimedShortSeq>("urgIn", m_urg);

        m_robotGps_val = new TimedDoubleSeq();
        m_robotGps = new DataRef<TimedDoubleSeq>(m_robotGps_val);
        m_gpsInIn = new InPort<TimedDoubleSeq>("gpsIn", m_robotGps);
        
//        m_button_val = new TimedBooleanSeq();
        m_button_val = new TimedBooleanSeq(new RTC.Time(0,0), new boolean[10]);
        m_button = new DataRef<TimedBooleanSeq>(m_button_val);
        m_buttonOutOut = new OutPort<TimedBooleanSeq>("buttonOut", m_button);

        m_compass_val = new TimedShort(new RTC.Time(0,0),(short)0);
        m_compass = new DataRef<TimedShort>(m_compass_val);
        m_compassOutOut = new OutPort<TimedShort>("compassOut", m_compass);
        
        m_gps_val = new TimedDoubleSeq();
        m_gps = new DataRef<TimedDoubleSeq>(m_gps_val);
        m_gpsOutOut = new OutPort<TimedDoubleSeq>("gpsOut", m_gps);
        
        m_accel_val = new TimedDoubleSeq(new RTC.Time(0,0), new double[3]);
        m_accel = new DataRef<TimedDoubleSeq>(m_accel_val);
        m_accelOutOut = new OutPort<TimedDoubleSeq>("accelOut", m_accel);
        
        m_status_val = new TimedString();
        m_status = new DataRef<TimedString>(m_status_val);
        m_statusOutOut = new OutPort<TimedString>("statusOut", m_status);
        
        m_tsukubaButton_val = new TimedUShort(new RTC.Time(0,0),(short)0);
        m_tsukubaButton = new DataRef<TimedUShort>(m_tsukubaButton_val);
        m_tsukubaButtonOutOut = new OutPort<TimedUShort>("tsukubaButtonOut", m_tsukubaButton);
        
        //RTC.Time arg0, int[] arg1, RTC.Velocity2D arg2, double[] arg3
        m_vel_val = new TimedVelocity2D(new RTC.Time(0,0), new int[0], new RTC.Velocity2D(0,0,0), new double[0]);
        m_vel = new DataRef<TimedVelocity2D>(m_vel_val);
        m_velOutOut = new OutPort<TimedVelocity2D>("velOut", m_vel);
        // </rtc-template>
    }

    /*!
     *
     * The initialize action (on CREATED->ALIVE transition)
     * formaer rtc_init_entry() 
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
    @Override
    protected ReturnCode_t onInitialize() {
        // Registration: InPort/OutPort/Service
        // <rtc-template block="registration">
        // Set InPort buffers
        addInPort("posIn", m_posInIn);
        addInPort("urgIn", m_urgInIn);
        addInPort("gpiIn", m_gpsInIn);
        
        // Set OutPort buffer
        addOutPort("buttonOut", m_buttonOutOut);
        addOutPort("compassOut", m_compassOutOut);
        addOutPort("gpsOut", m_gpsOutOut);
        addOutPort("accelOut", m_accelOutOut);
        addOutPort("statusOut", m_statusOutOut);
        addOutPort("tsukubaButtonOut", m_tsukubaButtonOutOut);
        addOutPort("velOut", m_velOutOut);
        // </rtc-template>
        bindParameter("iPhoneURL", m_iPhoneURL, "192.168.1.30");
        bindParameter("iPhonePort", m_iPhonePort, "57710");
        
        
        
		udpReceive.recievePort = 57110;
//		udpReceive.recievePort =m_iPhonePort.getValue();
		udpReceive.recieveFlag = false;
		Thread udpReceiveThread = new Thread(udpReceive);
		udpReceiveThread.start();

		dataGet.proxyFlag=true;
		dataGet.dataGetFlag=false;
		Thread dataGetThread = new Thread(dataGet);
		dataGetThread.start();
		
		mySend.sendIp="192.168.54.77";
		mySend.sendPort=12340;
		mySend.sendFlag=false;
		Thread sendThread= new Thread(mySend);
		sendThread.start();

		myReceive.recievePort=12340;
		myReceive.recieveFlag=false;
		Thread receiveThread= new Thread(myReceive);
		receiveThread.start();
		
        
        return super.onInitialize();
    }

    /***
     *
     * The finalize action (on ALIVE->END transition)
     * formaer rtc_exiting_entry()
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onFinalize() {
//        return super.onFinalize();
//    }

    /***
     *
     * The startup action when ExecutionContext startup
     * former rtc_starting_entry()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onStartup(int ec_id) {
//        return super.onStartup(ec_id);
//    }

    /***
     *
     * The shutdown action when ExecutionContext stop
     * former rtc_stopping_entry()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onShutdown(int ec_id) {
//        return super.onShutdown(ec_id);
//    }

    /***
     *
     * The activated action (Active state entry action)
     * former rtc_active_entry()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
    @Override
    protected ReturnCode_t onActivated(int ec_id) {
        return super.onActivated(ec_id);
    }

    /***
     *
     * The deactivated action (Active state exit action)
     * former rtc_active_exit()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
    @Override
    protected ReturnCode_t onDeactivated(int ec_id) {
		mySend.sendFlag=false;

        return super.onDeactivated(ec_id);
    }

    /***
     *
     * The execution action that is invoked periodically
     * former rtc_active_do()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
    @Override
    protected ReturnCode_t onExecute(int ec_id) {
    	
//    	System.out.println("aaaa");
    	
//		udpReceive.recievePort =m_iPhonePort.getValue();
//		System.out.println("PORT: "+udpReceive.recievePort);
    	
//    	System.out.println("groundAngle:"+udpReceive.groundAngle);
    	
    	udpReceive.recieveFlag=true;
    	
		
//		System.out.println("timerCounter: "+udpReceive.timerCounter);
    	
    	//position get
    	if(m_posInIn.isNew()){
    		m_posInIn.read();
//    		System.out.println("posData: "+m_robotPos.v.data[0]+","+m_robotPos.v.data[1]+","+m_robotPos.v.data[2]);
    		robotPosition=new Point3d(m_robotPos.v.data[0],m_robotPos.v.data[1],-m_robotPos.v.data[2]);
    		//iPhoneに送る	
//    		for(int i=0;i<m_robotPos.v.data.length;i++){
//    			mySend.sendMessage+=Double.toString(m_robotPos.v.data[i])+",";
//    		}
//    		mySend.sendMessage+="\n";
//    		mySend.sendFlag=true;
    		
    	}
		
		//LRF get
    	if(m_urgInIn.isNew()){
    		m_urgInIn.read();
//    		System.out.println("urgLength: "+m_urg.v.data.length);
    		
    		lrfDistance=new int[m_urg.v.data.length];
    		lrfDataRobot = new Point2d[m_urg.v.data.length];
//    		System.out.println("urg:");
    		mySend.sendMessage="";
    		for(int i=0;i<m_urg.v.data.length;i++){
//    			System.out.print(m_urg.v.data[i]+",");
    			mySend.sendMessage+=m_urg.v.data[i]+",";
    			lrfDistance[i]=(int)m_urg.v.data[i];
    			
    			double x=lrfDistance[i]*Math.cos(Math.PI*2/180*i);
    			double y=lrfDistance[i]*Math.sin(Math.PI*2/180*i);
    			lrfDataRobot[i]=new Point2d(x,y);
    			
    		}
//    		System.out.println();

    		mySend.sendMessage+="\n";
    		mySend.sendFlag=true;
    	}
//		int[] test=new int[10];
//		for(int i=0;i<test.length;i++){
//			test[i]=i*3;
//		}
//
//		mySend.sendMessage="";
//		for(int i=0;i<test.length;i++){
//			mySend.sendMessage+=Integer.toString(test[i])+",";
//		}
//		mySend.sendFlag=true;

		String dispSt ="";
		int testStatus = udpReceive.buttonStatus;
		boolean[] buttonBool =new boolean[10];
		for(int i=0;i<buttonBool.length;i++){
			buttonBool[i] = false;
		}
//		System.out.print("button: ");
		for(int i=0;i<buttonBool.length;i++){
			if(testStatus%2 ==1){
				buttonBool[i] = true;
			}else{
				buttonBool[i] = false;
			}
			testStatus = (testStatus-(testStatus%2))/2;
			if(buttonBool[i]){
//				System.out.print("o,");
				dispSt +="o, ";
			}else{
//				System.out.print("x,");
				dispSt +="x, ";
			}
		}
//		System.out.println();
		
		//mode analyze
		if(buttonBool[4])	controlMode="gamepad";
		if(buttonBool[6])	controlMode="joystick";
		if(buttonBool[8])	controlMode="angleMap";
		if(buttonBool[9])	controlMode="wayPoint";
		
//		System.out.println("mode:"+controlMode);
		
		//initialize
//		m_vel_val=new TimedDoubleSeq(new RTC.Time(0,0),new double[2]);
//		m_vel_val.data=new double[2];

//		m_vel_val.data[0]=0;
		m_vel_val.data.vx=0;
//		m_vel_val.data[1]=0;
		m_vel_val.data.va=0;
		
		if(controlMode.equals("gamepad")){
//			if(buttonBool[0])	m_vel_val.data[0]=0.5;
//			if(buttonBool[1])	m_vel_val.data[1]=0.7;
//			if(buttonBool[2])	m_vel_val.data[0]=-0.5;
//			if(buttonBool[3])	m_vel_val.data[1]=-0.7;
			
			if(buttonBool[0])	m_vel_val.data.vx=0.5;
			if(buttonBool[1])	m_vel_val.data.va=0.7;
			if(buttonBool[2])	m_vel_val.data.vx=-0.5;
			if(buttonBool[3])	m_vel_val.data.va=-0.7;

		}else if(controlMode.equals("joystick")){
//			System.out.println("angle:"+udpReceive.angleX+", "+udpReceive.angleY+", "+udpReceive.angleZ);
			if(buttonBool[5]){	//generate vel only when button6 is pushed.
				if(Math.abs(udpReceive.angleX)>20 && udpReceive.angleX<80){
//					m_vel_val.data[0]=(double)udpReceive.angleX/100.0;
					m_vel_val.data.vx=(double)udpReceive.angleX/100.0;
				}
				if(Math.abs(udpReceive.angleZ)>20 && udpReceive.angleZ<80){
//					m_vel_val.data[1]=(double)udpReceive.angleZ/100.0;
					m_vel_val.data.va=(double)udpReceive.angleZ/100.0;
				}
			}else{
//				m_vel_val.data[0]=0;
//				m_vel_val.data[1]=0;
				m_vel_val.data.vx=0;
				m_vel_val.data.va=0;
			}
		}else if(controlMode.equals("angleMap")){
			
	    	System.out.println("accel:"+udpReceive.angleX+","+udpReceive.angleY+","+udpReceive.angleZ);
	    	if(Math.abs(udpReceive.angleX)<7)	udpReceive.angleX=0;
	    	if(Math.abs(udpReceive.angleY)<7)	udpReceive.angleY=0;
	    	
//			m_vel_val.data[0]=(double)udpReceive.angleY/50.0;
//			m_vel_val.data[1]=(double)udpReceive.angleX/60.0;
			m_vel_val.data.vx=(double)udpReceive.angleY/50.0;
			m_vel_val.data.va=(double)udpReceive.angleX/60.0;
			
//			if(m_vel_val.data[0]>1)	m_vel_val.data[0]=1;
//			if(m_vel_val.data[0]<-1)	m_vel_val.data[0]=-1;
//			if(m_vel_val.data[1]>1)	m_vel_val.data[1]=1;
//			if(m_vel_val.data[1]<-1)	m_vel_val.data[1]=-1;
			if(m_vel_val.data.vx>1)	m_vel_val.data.vx=1;
			if(m_vel_val.data.vx<-1)	m_vel_val.data.vx=-1;
			if(m_vel_val.data.va>1)	m_vel_val.data.va=1;
			if(m_vel_val.data.va<-1)	m_vel_val.data.va=-1;
			
			
		}else if(controlMode.equals("wayPoint")){
//			if(robotPosition !=null){
//				if(udpReceive.wayPoint !=null){
//					if(robotVel !=null){
//						if(lrfDataRobot !=null){
//							VirtualTargetFollowing vtf=new VirtualTargetFollowing();
//							robotVel=vtf.getVelocity(robotPosition, udpReceive.wayPoint, robotVel, lrfDataRobot);
//							m_vel_val.data[0]=robotVel.x;
//							m_vel_val.data[1]=robotVel.y;
////							System.out.println("robotPosition"+robotPosition);
////							System.out.println("vtfVel: "+robotVel.x+", "+robotVel.y);
//						}
//					}
//				}
//			}
			
			if(robotPosition !=null){
				if(udpReceive.wayPoint !=null){
					udpReceive.robotPosition=robotPosition;
					if(robotVel !=null){
						VirtualTargetFollowing vtf=new VirtualTargetFollowing();
						System.out.println("ID: "+nowTargetID+"/"+udpReceive.wayPoint.length);
						if(nowTargetID==0){
							robotVel=vtf.getVelocityForiPhone(robotPosition, udpReceive.wayPoint[nowTargetID], new Point2d(0,0), robotVel);
							//if robot is near the target, taeget is update.
							double dist=udpReceive.wayPoint[nowTargetID].distance(new Point2d(robotPosition.x,robotPosition.y));
							System.out.println("distToTarget:"+dist);
							if(dist<300){
								nowTargetID++;
							}
							if(nowTargetID==udpReceive.wayPoint.length){
								controlMode = "gamepad";
								nowTargetID=0;
							}
						}else{
							robotVel=vtf.getVelocityForiPhone(robotPosition, udpReceive.wayPoint[nowTargetID], udpReceive.wayPoint[nowTargetID-1], robotVel);
							//if robot is near the target, taeget is update.
							double dist=udpReceive.wayPoint[nowTargetID].distance(new Point2d(robotPosition.x,robotPosition.y));
							System.out.println("distToTarget:"+dist);
							if(dist<300){
								nowTargetID++;
							}
							if(nowTargetID==udpReceive.wayPoint.length){
								controlMode = "gamepad";
								nowTargetID=0;
							}
						}
						System.out.println("WayPointVel:"+robotVel.x+","+robotVel.y);
						System.out.println("---------------------------------------------------");
//						m_vel_val.data[0]=robotVel.x;
//						m_vel_val.data[1]=robotVel.y;
						m_vel_val.data.vx=robotVel.x;
						m_vel_val.data.va=robotVel.y;
					}
				}
			}
		}else{
//			m_vel_val.data[0]=0;
//			m_vel_val.data[1]=0;
			m_vel_val.data.vx=0;
			m_vel_val.data.va=0;
		}
		
//		if(m_vel_val.data[0]>1)	m_vel_val.data[0]=1;
//		if(m_vel_val.data[0]<-1)	m_vel_val.data[0]=-1;
//		if(m_vel_val.data[1]>1)	m_vel_val.data[0]=1;
//		if(m_vel_val.data[1]<-1)	m_vel_val.data[0]=-1;
		if(m_vel_val.data.vx>1)	m_vel_val.data.vx=1;
		if(m_vel_val.data.vx<-1)	m_vel_val.data.vx=-1;
		if(m_vel_val.data.va>1)	m_vel_val.data.va=1;
		if(m_vel_val.data.va<-1)	m_vel_val.data.va=-1;
		
//		m_vel_val.data[0]=0.5;
		
//		System.out.println("vel: "+m_vel_val.data[0]+","+m_vel_val.data[1]);		
		 m_velOutOut.write();

		m_tsukubaButton_val=new TimedUShort();
		if(buttonBool[7]){
			m_tsukubaButton_val.data=1;
			m_tsukubaButtonOutOut.write();
		}else if(buttonBool[8]){
			m_tsukubaButton_val.data=3;
			m_tsukubaButtonOutOut.write();
		}else{
			m_tsukubaButton_val.data=0;
			m_tsukubaButtonOutOut.write();
		}
//		System.out.println("tsukubaButton:"+m_tsukubaButton_val.data);
		
		
		//button dataを出力
		m_button_val.data=new boolean[buttonBool.length];
		for(int i=0;i<buttonBool.length;i++){
			m_button_val.data[i]=buttonBool[i];
		}
		m_buttonOutOut.write();
		
		//compass dataを出力
		
		
		//accelデータを出力
		m_accel_val.data=new double[3];
		m_accel_val.data[0]=Math.toRadians(udpReceive.angleX);	//送信時に間違ってdegree変換をしちゃってる。
		m_accel_val.data[1]=Math.toRadians(udpReceive.angleY);	//送信時に間違ってdegree変換をしちゃってる。
		m_accel_val.data[2]=Math.toRadians(udpReceive.angleZ);	//送信時に間違ってdegree変換をしちゃってる。
//		System.out.println("accel:"+m_accel_val.data[0]+", "+m_accel_val.data[1]+", "+m_accel_val.data[2]);
		m_accelOutOut.write();
    	
        return super.onExecute(ec_id);
    }

    /***
     *
     * The aborting action when main logic error occurred.
     * former rtc_aborting_entry()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//  @Override
//  public ReturnCode_t onAborting(int ec_id) {
//      return super.onAborting(ec_id);
//  }

    /***
     *
     * The error action in ERROR state
     * former rtc_error_do()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    public ReturnCode_t onError(int ec_id) {
//        return super.onError(ec_id);
//    }

    /***
     *
     * The reset action that is invoked resetting
     * This is same but different the former rtc_init_entry()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onReset(int ec_id) {
//        return super.onReset(ec_id);
//    }

    /***
     *
     * The state update action that is invoked after onExecute() action
     * no corresponding operation exists in OpenRTm-aist-0.2.0
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onStateUpdate(int ec_id) {
//        return super.onStateUpdate(ec_id);
//    }

    /***
     *
     * The action that is invoked when execution context's rate is changed
     * no corresponding operation exists in OpenRTm-aist-0.2.0
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onRateChanged(int ec_id) {
//        return super.onRateChanged(ec_id);
//    }
//
	// Configuration variable declaration
	// <rtc-template block="config_declare">
    /*!
     * 
     * - Name:  iPhoneURL
     * - DefaultValue: 192.168.1.30
     */
    protected StringHolder m_iPhoneURL = new StringHolder();
    /*!
     * 
     * - Name:  iPhonePort
     * - DefaultValue: 57710
     */
    protected IntegerHolder m_iPhonePort = new IntegerHolder();
	// </rtc-template>

    // DataInPort declaration
    // <rtc-template block="inport_declare">
    protected TimedDoubleSeq m_robotPos_val;
    protected DataRef<TimedDoubleSeq> m_robotPos;
    /*!
     */
    protected InPort<TimedDoubleSeq> m_posInIn;

    protected TimedShortSeq m_urg_val;
    protected DataRef<TimedShortSeq> m_urg;
    /*!
     */
    protected InPort<TimedShortSeq> m_urgInIn;

    protected TimedDoubleSeq m_robotGps_val;
    protected DataRef<TimedDoubleSeq> m_robotGps;
    /*!
     */
    protected InPort<TimedDoubleSeq> m_gpsInIn;

    
    // </rtc-template>

    // DataOutPort declaration
    // <rtc-template block="outport_declare">
    protected TimedBooleanSeq m_button_val;
    protected DataRef<TimedBooleanSeq> m_button;
    /*!
     */
    protected OutPort<TimedBooleanSeq> m_buttonOutOut;

    protected TimedShort m_compass_val;
    protected DataRef<TimedShort> m_compass;
    /*!
     */
    protected OutPort<TimedShort> m_compassOutOut;

    protected TimedDoubleSeq m_gps_val;
    protected DataRef<TimedDoubleSeq> m_gps;
    /*!
     */
    protected OutPort<TimedDoubleSeq> m_gpsOutOut;

    protected TimedDoubleSeq m_accel_val;
    protected DataRef<TimedDoubleSeq> m_accel;
    /*!
     */
    protected OutPort<TimedDoubleSeq> m_accelOutOut;

    protected TimedString m_status_val;
    protected DataRef<TimedString> m_status;
    /*!
     */
    protected OutPort<TimedString> m_statusOutOut;

    protected TimedUShort m_tsukubaButton_val;
    protected DataRef<TimedUShort> m_tsukubaButton;
    /*!
     */
    protected OutPort<TimedUShort> m_tsukubaButtonOutOut;

    protected TimedVelocity2D m_vel_val;
    protected DataRef<TimedVelocity2D> m_vel;
    /*!
     */
    protected OutPort<TimedVelocity2D> m_velOutOut;

    
    // </rtc-template>

    // CORBA Port declaration
    // <rtc-template block="corbaport_declare">
    
    // </rtc-template>

    // Service declaration
    // <rtc-template block="service_declare">
    
    // </rtc-template>

    // Consumer declaration
    // <rtc-template block="consumer_declare">
    
    // </rtc-template>
    
    
    //ユーザ定義変数
	OSCIPhoneController udpReceive = new OSCIPhoneController();
	URLDataDownloader dataGet=new URLDataDownloader();
	
	UDPSendTerminalSimple mySend= new UDPSendTerminalSimple();
	UDPReceiveTerminalSimple myReceive =new UDPReceiveTerminalSimple();
	Point2d robotVel=new Point2d(0,0);
	int[] lrfDistance=null;
	
	Point2d[] lrfDataRobot=null;
	
	int nowTargetID=0;
	
	//control mode
	String controlMode="gamepad";

	Point3d robotPosition=null;


}
